""" Sink - Arch Linux AUR Package Installer """
__version__ = '0.1.0b2'
__author__ = 'Mike Shultz'
__copyright__ = "Copyright 2018 Mike Shultz"
__license__ = 'GPLv3'
__maintainer__ = __author__
__email__ = 'mike@mikeshultz.com'
__status__ = 'Alpha'
